﻿/*
 * Program.cs - Main
 * 
 * ShakeIt - Mutation engine for fuzzing
 * Jeremy Brown (jbrown3264/gmail)
 * Nov 2015
 * 
 */

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ShakeIt
{
    public class Program
    {
        public static List<string> BlackList = new List<string>();

        static int Main(string[] args)
        {
            int ret = DoWork(args);
            return 0;
        }

        public static int DoWork(string[] args)
        {
            if (args.Count() < 3)
            {
                Misc.Options();
                return 0;
            }

            string outputDirectory = args[0];
            string inputFilename = args[1];
            uint numberTestCases = Convert.ToUInt32(args[2]); // if not given a significantly complex input, this number will be much smaller

            uint numberCreatedTestCases = 0;

            if (!Directory.Exists(outputDirectory))
            {
                Console.WriteLine("\nError: DoWork() - {0} is not a directory or does not exist at all", outputDirectory);
                return -1;
            }

            Console.WriteLine();
            Console.WriteLine("Input File:          {0}", inputFilename);
            Console.WriteLine("Output Directory:    {0}\n", outputDirectory);
            Console.WriteLine("Testcases Requested: {0}\n", numberTestCases);

            /*
             * Web content is fileType 1, PDFs are 2
             */
            int fileType = 1;
            if (inputFilename.EndsWith(".pdf"))
            {
                fileType = 2;
            }

            /*
             * Create a new Task (.NET thread) for each iteration
             */
            var taskList = new List<Task>();

            for (uint i = 0; i < numberTestCases; i++)
            {
                var task = Task.Factory.StartNew(() =>
                {
                    if (fileType == 1)
                    {
                        BlackList = Misc.WebBlackList;

                        string stringInput = Read.InputAsString(inputFilename, outputDirectory);

                        if (stringInput == null)
                        {
                            Console.WriteLine("\nError: DoWork() - couldn't read strings from {0}", inputFilename);
                            Thread.Sleep(10000000);
                        }

                        /*
                         * For each token id, keep shaking the input into output
                         * We must keep mutating input as there are multiple kinds
                         * of tokens to change in each run
                         */
                        string stringOutput = stringInput;

                        foreach (char id in Misc.TokenIds)
                        {
                            stringOutput = Shake.It(id, stringOutput).ToString();
                        }

                        //string outputSha1 = Misc.GetSha1(Encoding.ASCII.GetBytes(outputString.ToString()));
                        //Console.WriteLine("outputString SHA1: {0}\npreviousOutputString SHA1: {1}\n\n", outputSha1, previousOutputSha1);

                        if (stringOutput == string.Empty)
                        {
                            Console.WriteLine("Error: Shake.It() returned an empty string");
                        }

                        //string outputString = inputString;

                        string outputFilePath = outputDirectory + "\\" + Misc.GetSha1(Encoding.ASCII.GetBytes(stringOutput.ToString())) + ".html";

                        //Console.WriteLine("outputFilePath = {0}\n", outputFilePath);

                        try
                        {
                            if (!File.Exists(outputFilePath))
                            {
                                File.WriteAllText(outputFilePath, stringOutput.ToString());
                                numberCreatedTestCases++;
                            }
                        }
                        catch (Exception error)
                        {
                            // we expect this error
                            //Console.WriteLine("Error: WriteAllText() - {0}", error.Message);
                        }
                    }

                    if (fileType == 2)
                    {
                        BlackList = Misc.PDFBlackList;

                        byte[] byteInput = Read.InputAsBytes(inputFilename, outputDirectory);

                        if (byteInput == null)
                        {
                            Console.WriteLine("\nError: DoWork() - couldn't read bytes from {0}", inputFilename);
                        }

                        if (byteInput == null)
                        {
                            Console.WriteLine("\nError: DoWork() - couldn't read strings from {0}", inputFilename);
                            Thread.Sleep(10000000);
                        }

                        /*
                         * For each token id, keep shaking the input into output
                         * We must keep mutating input as there are multiple kinds
                         * of tokens to change in each run
                         */
                        byte[] byteOutput = null;
                        string byteString = Encoding.Unicode.GetString(byteInput);

                        foreach (char id in Misc.TokenIds)
                        {
                            byteOutput = Encoding.Unicode.GetBytes(Shake.It(id, byteString).ToString());
                        }

                        if (byteOutput == null)
                        {
                            Console.WriteLine("Error: Shake.It() returned null");
                        }
                        string outputFilePath = outputDirectory + "\\" + Misc.GetSha1(byteOutput) + ".pdf";

                        //Console.WriteLine("outputFilePath = {0}\n", outputFilePath);

                        try
                        {
                            if (!File.Exists(outputFilePath))
                            {
                                File.WriteAllBytes(outputFilePath, byteOutput);
                                numberCreatedTestCases++;
                            }
                        }
                        catch (Exception error)
                        {
                            // we expect this error
                            //Console.WriteLine("Error: WriteAllText() - {0}", error.Message);
                        }
                    }
                });

                taskList.Add(task);
            }

            Task.WhenAll(taskList).Wait();

            Console.WriteLine("> Unique Testcases Created: {0} ({1}% of target)\n", numberCreatedTestCases, Math.Round((double)(numberCreatedTestCases * 100/numberTestCases), 1));

            return 0;
        }
    }
}
