﻿/*
 * Shake.cs - Mutation Core
 * 
 * ShakeIt - Mutation engine for fuzzing
 * Jeremy Brown (jbrown3264/gmail)
 * Nov 2015
 * 
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ShakeIt
{
    public class Shake
    {
        public static StringBuilder It(char tokenId, string input)
        {
            List<TokenData> tokens = Token.Find(tokenId, input);

            /*
             * It takes two to tango
             * If we don't get at least two tokens, return input and try the next token
             */
            if (tokens.Count < 2)
            {
                return new StringBuilder(input);
            }

            /*
             * Return a list of shaken tokens
             */
            List<TokenData> shakenTokens = Shuffle(tokens);
            
            var output = new StringBuilder();

            /*
             * Use new positions and lengths of shuffled tokens to build output file
             */
            int currentPosition = 0;
            foreach (TokenData token in tokens)
            {
                output.Append(input.Substring(currentPosition, token.Position - currentPosition));

                int tokenIndex = tokens.IndexOf(token);
                TokenData newTokenIndex = shakenTokens.ElementAt(tokenIndex);

                output.Append(input.Substring(newTokenIndex.Position, newTokenIndex.Length));

                currentPosition = token.Position + token.Length;
            }

            /*
             * Keep adding the contents other than token data if we're not at the end of input
             */
            if (currentPosition < input.Length)
            {
                output.Append(input.Substring(currentPosition, input.Length - currentPosition));
            }

            return output;
        }

        public static List<TokenData> Shuffle(List<TokenData> tokenList)
        {
            bool doit = false;
            List<TokenData> shuffledTokens = new List<TokenData>();
            Random random = new Random(Guid.NewGuid().GetHashCode());

            /*
             * Make an int list for all of the tokens
             */
            List<int> range = Enumerable.Range(0, tokenList.Count).ToList();

            int randomIndex = random.Next(1, tokenList.Count);
            int position = range.ElementAt(randomIndex);

            shuffledTokens.Add(tokenList[position]);
            range.RemoveAt(randomIndex);

            /*
             * Iterate from the end, choose an index at random, adding it to the
             * shuffled list and then removing it from the pool until we've
             * exhausted the list
             */
            for (int i = (tokenList.Count - 2); i >= 0; i--)
            {
                randomIndex = random.Next(0, i + 1);
                position = range.ElementAt(randomIndex);

                shuffledTokens.Add(tokenList[position]);
                range.RemoveAt(randomIndex);
            }

            return shuffledTokens;
        }
    }
}
