﻿/*
 * UnitTest1.cs - Mutation Test
 * 
 * ShakeIt - Mutation engine for fuzzing
 * Jeremy Brown (jbrown3264/gmail)
 * Nov 2015
 * 
 */

using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ShakeIt;

namespace Tests
{
    [TestClass]
    public class UnitTests
    {
        [TestMethod]
        public void TestFindTokens()
        {
            string input = "<1><22>";

            List<TokenData> tokens = Token.Find('<', input);

            Assert.AreEqual(2, tokens.Count);

            Assert.AreEqual(1, tokens.ElementAt(0).Position);
            Assert.AreEqual(1, tokens.ElementAt(0).Length);
            Assert.AreEqual(4, tokens.ElementAt(1).Position);
            Assert.AreEqual(2, tokens.ElementAt(1).Length);
        }

        [TestMethod]
        public void TestShakeIt()
        {
            string input = "<1><22><333><4444>";

            string output = Shake.It('<', input).ToString();

            Assert.AreEqual(input.Length, output.Length);
            Assert.AreNotEqual(input, output);
        }

        [TestMethod]
        public void TestShuffle()
        {
            List<TokenData> tdList = new List<TokenData>();

            tdList.Add(new TokenData(1, 0));
            tdList.Add(new TokenData(2, 0));
            tdList.Add(new TokenData(3, 0));
            tdList.Add(new TokenData(4, 0));
            tdList.Add(new TokenData(5, 0)); 

            List<TokenData> shuffled = Shake.Shuffle(tdList);

            CollectionAssert.AreEquivalent(tdList, shuffled);
            CollectionAssert.AreNotEqual(tdList, shuffled);

        }
    }
}
